//=============================================================================
// TilingSpriteTextureFix.js (V2)
//=============================================================================

/*
 * @plugindesc Fixes the tiling sprite texture caching and destroying so that no memory leaks happen when switching a TilingSprite's bitmap (happens also when a parallax on a map is present and the menu is called/closed repeatedly or a transfer is performed).
 */

(function() {

TilingSprite.prototype.uncacheTilingTexture = function() 
{
    if (this.tilingTexture && this.tilingTexture.canvasBuffer)
    {
        PIXI.BaseTextureCache[this.tilingTexture.canvasBuffer.canvas._pixiId].destroy(true);
        delete PIXI.BaseTextureCache[this.tilingTexture.canvasBuffer.canvas._pixiId];
    }
}

TilingSprite.prototype._refresh = function() 
{
    var frame = this._frame.clone();
    if (frame.width === 0 && frame.height === 0 && this._bitmap)
    {
        frame.width = this._bitmap.width;
        frame.height = this._bitmap.height;
    }
    var lastTrim = this.texture.trim;
    this.texture.trim = frame;
    this.texture.setFrame(frame);
    this.texture.trim = lastTrim;
    
    this.uncacheTilingTexture();

    this.tilingTexture = null;
};

TilingSprite.prototype.uncacheAndDestroyTilingTexture = function()
{
    this.uncacheTilingTexture();
    var tilingTexture = this.tilingTexture;
    
    if (tilingTexture)
    {
        tilingTexture.destroy(true);
        tilingTexture = null;
    }
}

var _Spriteset_Map_createParallax = Spriteset_Map.prototype.createParallax;
Spriteset_Map.prototype.createParallax = function()
{
    _Spriteset_Map_createParallax.call(this);
    Scene_Menu.parallax = this._parallax;
    Game_Player.parallax = this._parallax;
};

var _Scene_Menu_create = Scene_Menu.prototype.create;
Scene_Menu.prototype.create = function() 
{
    _Scene_Menu_create.call(this);
    
    var parallax = Scene_Menu.parallax;
    if (parallax)
    {
        parallax.uncacheAndDestroyTilingTexture();
        Scene_Menu.parallax = null;
    }
};

var _Game_Player_performTransfer = Game_Player.prototype.performTransfer;
Game_Player.prototype.performTransfer = function()
{
    _Game_Player_performTransfer.call(this);
    
    var parallax = Game_Player.parallax;
    if (parallax)
    {
        parallax.uncacheAndDestroyTilingTexture();
        Game_Player.parallax = null;
    }
}

})();