//=============================================================================
// MOG_LimitedVisibility.js
//=============================================================================
/*:
 * @plugindesc (v1.0) Apresenta uma esfera de luz circulando ao redor do personagem.
 * @author Moghunter
 *
 * @param Visible X-Axis
 * @desc Posição X-Axis area.
 * @default 0
 *
 * @param Visible Y-Axis
 * @desc Posição Y-Axis area.
 * @default -24
 *
 * @param Move Range
 * @desc Area de movimento do círculo.
 * @default 60
 *
 * @param Object Frames
 * @desc Quantidade de Frames de animação do objeto
 * @default 3
 *
 * @param Object An Speed
 * @desc Velocidade de animação do objeto.
 * @default 6
 *
 * @param Object X-Axis
 * @desc Definição X-Axis do objeto.
 * @default 0
 *
 * @param Object Y-Axis
 * @desc Definição Y-Axis do objeto.
 * @default 0
 *
 * @help  
 * =============================================================================
 * +++ MOG - Limited Visibility (v1.0) +++
 * By Moghunter 
 * https://atelierrgss.wordpress.com/
 * =============================================================================
 * Apresenta uma esfera de luz circulando ao redor do personagem.
 * =============================================================================
 * PLUGIN COMMAND
 * =============================================================================
 * Utilize o comando abaixo para ativar a esfera de luz.
 *
 * limitedvisibility : MODE : FILENAME1 : FILENAME2 : OPACITY
 *
 * MODE -> 0 - Standstill   1 - Circulating
 * FILENAME1 -> Sprite do Objecto de luz.
 * FILENAME2 -> Sprite da área visível.
 * OPACITY -> Opacidade dos sprites.
 *
 * EG
 *
 * spherelight : 1 : LightA : LightB : 220
 *
 * Nota - As imagens deverão ser gravadas na pasta /img/system/
 *
 * -----------------------------------------------------------------------------
 * Para cancelar o efeito use o comando abaixo.
 *
 * clearvisibility
 *
 */

//=============================================================================
// ** PLUGIN PARAMETERS
//=============================================================================
　　var Imported = Imported || {};
　　Imported.MOG_CharTrailEffect = true;
　　var Moghunter = Moghunter || {}; 

  　Moghunter.parameters = PluginManager.parameters('MOG_CharTrailEffect');  
    Moghunter.lightSphereX = Number(Moghunter.parameters['Area X-Axis'] || 0);
	
//=============================================================================
// ** Game_Interpreter
//=============================================================================	

//==============================
// * PluginCommand
//==============================
var _mog_trailEffect_pluginCommand = Game_Interpreter.prototype.pluginCommand
Game_Interpreter.prototype.pluginCommand = function(command, args) {
	_mog_trailEffect_pluginCommand.call(this,command, args)

	return true;
};

//=============================================================================
// ** Spriteset Map
//=============================================================================

//==============================
// * create Characters
//==============================
var _mog_trailEffect_sprmap_createCharacters = Spriteset_Map.prototype.createCharacters;
Spriteset_Map.prototype.createCharacters = function() {
	_mog_trailEffect_sprmap_createCharacters.call(this);
	this.createTrailSprites();
};

//==============================
// * create Trail Sprites
//==============================
Spriteset_Map.prototype.createTrailSprites = function() {
	this._charTrail = [];
	for (var i = 0; i < this._characterSprites.length; i++) {
	     this._charTrail[i] = new CharTrail(this._characterSprites[i]);
	     this._tilemap.addChild(this._charTrail[i]);
	};
};




//=============================================================================
// ** Char Trail
//=============================================================================
function CharTrail() {
    this.initialize.apply(this, arguments);
};
CharTrail.prototype = Object.create(Sprite.prototype);
CharTrail.prototype.constructor = CharTrail;

//==============================
// * Initialize
//==============================
CharTrail.prototype.initialize = function(sprite) {
    Sprite.prototype.initialize.call(this);
    this._sprite = sprite;
	this._character = this._sprite._character;
	this._tilesetId = 0;
};

//==============================
// * refresh Bitmap
//==============================
CharTrail.prototype.refreshBitmap = function() {
     this._tilesetId = $gameMap.tilesetId();
     this._tileId = this._character.tileId();
     this._characterName = this._character.characterName();
     this._characterIndex = this._character.characterIndex();
     if (this._tileId > 0) {
         this.setTileBitmap();
     } else {
         this.setCharacterBitmap();
     }
};


//==============================
// * set Tile Bitmap
//==============================
CharTrail.prototype.setTileBitmap = function() {
    this.bitmap = this.tilesetBitmap(this._tileId);
};

//==============================
// * set Character Bitmap
//==============================
CharTrail.prototype.setCharacterBitmap = function() {
    this.bitmap = ImageManager.loadCharacter(this._characterName);
    this._isBigCharacter = ImageManager.isBigCharacter(this._characterName);
};

//==============================
// * is Image Changed
//==============================
CharTrail.prototype.isImageChanged = function() {
    return (this._tilesetId !== $gameMap.tilesetId() ||
            this._tileId !== this._character.tileId() ||
            this._characterName !== this._character.characterName() ||
            this._characterIndex !== this._character.characterIndex());
};

//==============================
// * Update
//==============================
CharTrail.prototype.update = function() {
    Sprite.prototype.update.call(this);
	if (this.isImageChanged()) {this.refreshBitmap()};
};