Window_ShopStatus.prototype.drawActorSvActor = function(actor, poseIndex, frameIndex, x, y) {
    this.drawSvActor(actor.battlerName(), poseIndex, this._frameOrder[frameIndex], x, y);
};

Window_ShopStatus.prototype.drawSvActor = function(battlerName, poseIndex, frameIndex, x, y) {
    var bitmap = ImageManager.loadSvActor(battlerName);
    var pw = bitmap.width / 9;
    var ph = bitmap.height / 6;
    var sx = (poseIndex % 3 * pw * 3) + (frameIndex * pw);
    var sy = Math.floor(poseIndex / 3) * ph;
    this.contents.blt(bitmap, sx, sy, pw, ph, x, y - 67); // change this 25 (Default 0)
};

Window_ShopStatus.prototype.initialize = function(x, y, width, height) {
    Window_Base.prototype.initialize.call(this, x, y, width, height);
    this._item = null;
    this._pageIndex = 0;
    this._frameCount = 0;
    this._frameIndex = 0;
    this._frameOrder = [1, 0, 1, 2];
    this.refresh();
};

Window_ShopStatus.prototype.refresh = function() {
    this.contents.clear();
    if (this._item) {
        var x = this.textPadding();
        this.drawPossession(x, 0);
        if (this.isEquipItem()) {
            this.drawEquipInfo(x, this.lineHeight() * 1); // change this 1 (Default 2)
        }
    }
};

Window_ShopStatus.prototype.drawEquipInfo = function(x, y) {
    var members = this.statusMembers();
    for (var i = 0; i < members.length; i++) {
        this.drawActorEquipInfo(x, y + this.lineHeight() * (i * 2.8), members[i]); //change this 2.8 (Default 2.4)
    }
};

Window_ShopStatus.prototype.drawPossession = function(x, y) {
    var width = this.contents.width - this.textPadding() - x;
    var possessionWidth = this.textWidth('0000');
    this.changeTextColor(this.systemColor());
    this.drawText(TextManager.possession, x, y, width - possessionWidth, 'right');
    this.resetTextColor();
    this.drawText($gameParty.numItems(this._item), x, y, width, 'right');
};

Window_ShopStatus.prototype.drawActorEquipInfo = function(x, y, actor) {
    var enabled = actor.canEquip(this._item);
    this.changePaintOpacity(enabled);
    this.resetTextColor();
    var item1 = this.currentEquippedItem(actor, this._item.etypeId);
    if (enabled) {
        this.drawActorParamChange(x + 82, y + this.lineHeight(), actor, item1);
        this.drawActorSvActor(actor, 5, this._frameIndex, x, y);
    }
    else
    {
        this.drawActorSvActor(actor, 0, this._frameIndex, x, y);
    }
    if(item1)
    {
      this.drawItemName(item1, x + 82, y);
    }
    else
    {
      this.resetTextColor();
      this.changePaintOpacity(false);
      this.drawText("Empty", x + 82, y);
    }
    this.changePaintOpacity(true);
};

Window_ShopStatus.prototype.drawActorParamChange = function(x, y, actor, item1) {
    var width = this.contents.width - this.textPadding() - x;
    if(DataManager.isWeapon(this._item))
    {
      var change1 = this._item.params[2] - (item1 ? item1.params[2] : 0);
      var change2 = this._item.params[4] - (item1 ? item1.params[4] : 0);
      this.changeTextColor(this.normalColor());
      this.drawText("Atk:", x, y, width);
      this.drawText("MAtk:", x + 130, y, width);
      this.changeTextColor(this.paramchangeTextColor(change1));
      this.drawText((change1 > 0 ? '+' : '') + change1, x + 75, y, width);
      this.changeTextColor(this.paramchangeTextColor(change2));
      this.drawText((change2 > 0 ? '+' : '') + change2, x + 210, y, width);
    }
    else
    {
      var change1 = this._item.params[3] - (item1 ? item1.params[3] : 0);
      var change2 = this._item.params[5] - (item1 ? item1.params[5] : 0);
      this.changeTextColor(this.normalColor());
      this.drawText("PDef:", x, y, width);
      this.drawText("MDef:", x + 130, y, width);
      this.changeTextColor(this.paramchangeTextColor(change1));
      this.drawText((change1 > 0 ? '+' : '') + change1, x + 75, y, width);
      this.changeTextColor(this.paramchangeTextColor(change2));
      this.drawText((change2 > 0 ? '+' : '') + change2, x + 210, y, width);
    }
};

Window_ShopStatus.prototype.update = function() {
    Window_Base.prototype.update.call(this);
    this._frameCount++;
    if(this._frameCount > 80)
    {
      this._frameCount = 0;
    }
    if(Math.floor(this._frameCount / 20) != this._frameIndex)
    {
      this._frameIndex = this._frameIndex + 1 > 3 ? 0 : this._frameIndex + 1;
      this.refresh();
    }
    this.updatePage();
};

