//-----------------------------------------------------------------------------
// Window_BattleStatus
//
// The window for displaying the status of party members on the battle screen.



Window_BattleStatus.prototype.initialize = function() {
    var width = this.windowWidth();
    var height = 244;
    var x = Graphics.boxWidth - width;
    var y = Graphics.boxHeight - height;
    Window_Selectable.prototype.initialize.call(this, x, y, width, height);
    this.refresh();
    this.openness = 0;
    this.windowskin = ImageManager.loadSystem('Battle');
};

Window_BattleStatus.prototype.windowWidth = function() {
    return Graphics.boxWidth - 500;
};

Window_BattleStatus.prototype.windowHeight = function() {
    return this.fittingHeight(this.numVisibleRows()) ;
};

Window_BattleStatus.prototype.numVisibleRows = function() {
    return 4;
};

Window_BattleStatus.prototype.maxItems = function() {
    return $gameParty.battleMembers().length;
};

Window_BattleStatus.prototype.refresh = function() {
    this.contents.clear();
    this.drawAllItems();
};

Window_BattleStatus.prototype.itemHeight = function() {
    return 52;
};
Window_BattleStatus.prototype.drawItem = function(index) {
    var actor = $gameParty.battleMembers()[index];
    this.drawBasicArea(this.basicAreaRect(index), actor);
    this.drawGaugeArea(this.gaugeAreaRect(index), actor);
};

Window_BattleStatus.prototype.basicAreaRect = function(index) {
    var rect = this.itemRectForText(index);
    rect.width -= this.gaugeAreaWidth() + 20;

    return rect;
};

Window_BattleStatus.prototype.gaugeAreaRect = function(index) {
    var rect = this.itemRectForText(index);
    rect.x += rect.width - this.gaugeAreaWidth();
    rect.y+=20;
    rect.width = this.gaugeAreaWidth();

    return rect;
};

Window_BattleStatus.prototype.gaugeAreaWidth = function() {
    return 330;
};

Window_BattleStatus.prototype.drawBasicArea = function(rect, actor) {
this.contents.fontSize = 22;
this.contents.fontFace = 'Epilog';
    this.drawActorName(actor, rect.x + 220, rect.y, 150);
    this.drawActorIcons(actor, rect.x + 20, rect.y+20, rect.width - 5);
    
};

Window_BattleStatus.prototype.drawGaugeArea = function(rect, actor) {
    if ($dataSystem.optDisplayTp) {
        this.drawGaugeAreaWithTp(rect, actor);
    } else {
        this.drawGaugeAreaWithoutTp(rect, actor);
    }
};

Window_BattleStatus.prototype.drawGaugeAreaWithTp = function(rect, actor) {
var oldfontsize = this.contents.fontSize;
this.contents.fontSize = 20;
this.contents.fontFace = 'Dadhand';

    this.drawActorHp(actor, rect.x + 0, rect.y, 150);
    this.drawActorMp(actor, rect.x + 155, rect.y, 150);
this.contents.fontFace = 'Dadhand';
    this.drawActorTp(actor, rect.x + 155, rect.y-25, 150);
    this.contents.fontSize = oldfontsize;

    
};

Window_BattleStatus.prototype.drawGaugeAreaWithoutTp = function(rect, actor) {
    this.drawActorHp(actor, rect.x + 0, rect.y, 201);
    this.drawActorMp(actor, rect.x + 216,  rect.y, 114);
   
};

Window_BattleStatus.prototype.drawActorHp = function(actor, x, y, width) {
    width = width || 186;
    var oldfontsize = this.contents.fontSize;
    var color1 = this.hpGaugeColor1();
    var color2 = this.hpGaugeColor2();
    this.drawGauge(x, y, width, actor.hpRate(), color1, color2);
    this.changeTextColor(this.textColor(4));
    this.contents.fontSize = 15;
    this.drawText(TextManager.hpA, x, y, 44);
    this.contents.fontSize = oldfontsize;
    this.drawCurrentAndMax(actor.hp, actor.mhp, x, y, width,
                           this.hpColor(actor), this.normalColor());
                           
};

Window_BattleStatus.prototype.drawActorMp = function(actor, x, y, width) {
    width = width || 186;
    var oldfontsize = this.contents.fontSize;
    var color1 = this.mpGaugeColor1();
    var color2 = this.mpGaugeColor2();
     this.contents.fontSize = 10;
    this.drawGauge(x, y, width, actor.mpRate(), color1, color2);
    this.changeTextColor(this.textColor(3));
    this.drawText(TextManager.mpA, x, y, 44);
    this.contents.fontSize = oldfontsize;
    this.drawCurrentAndMax(actor.mp, actor.mmp, x, y, width,
                           this.mpColor(actor), this.normalColor());
};

Window_BattleStatus.prototype.drawActorTp = function(actor, x, y, width) {
    width = width || 96;
        var oldfontsize = this.contents.fontSize;
    var color1 = this.tpGaugeColor1();
    var color2 = this.tpGaugeColor2();
    this.contents.fontSize = 15;
        
    this.drawGauge(x, y, width, actor.tpRate(), color1, color2);
     this.changeTextColor(this.textColor(7));
    this.drawText(TextManager.tpA, x, y, 44);
    this.changeTextColor(this.tpColor(actor));
        this.contents.fontSize = oldfontsize;
             this.contents.fontSize = 0;
    this.drawText(actor.tp, x + width - 64, y, 64, 'right');
};

Scene_Battle.prototype.updateWindowPositions = function() {
};

Scene_Battle.prototype.createSkillWindow = function() {
    var wy = this._helpWindow.y + this._helpWindow.height;
    var wh = this._statusWindow.y - wy;
    this._skillWindow = new Window_BattleSkill(0, 360, Graphics.boxWidth/2 + 50, wh);
    this._skillWindow.setHelpWindow(this._helpWindow);
    this._skillWindow.setHandler('ok',     this.onSkillOk.bind(this));
    this._skillWindow.setHandler('cancel', this.onSkillCancel.bind(this));
    this.addWindow(this._skillWindow);
};

Scene_Battle.prototype.createItemWindow = function() {
    var wy = this._helpWindow.y + this._helpWindow.height;
    var wh = this._statusWindow.y - wy;
    this._itemWindow = new Window_BattleItem(0, 360, Graphics.boxWidth/2 + 50, wh);
    this._itemWindow.setHelpWindow(this._helpWindow);
    this._itemWindow.setHandler('ok',     this.onItemOk.bind(this));
    this._itemWindow.setHandler('cancel', this.onItemCancel.bind(this));
    this.addWindow(this._itemWindow);
};

Scene_Battle.prototype.start = function() {
    Scene_Base.prototype.start.call(this);
    this.createFadeSprite(false);
    this._fadeSprite.opacity = 255;
    BattleManager.playBattleBgm();
    BattleManager.startBattle();
};
